@@ -1,24 +1,45 @@
-##################################################################################
-#
-# ddrlib - Collection of functions used in DDR-Python scripts
-#
-# ddrlib functions interact with device management interfaces including
-# NETCONF, ssh, cli, NETCONF RFC5277 notificaitons.
-# Scripts using ddrlib functions can perform multiple operations including:
-#
-#    * Triggering execution when specific Syslog messages are generated
-#    * Triggering execution by executing show commands, extracting parameters and triggering on values
-#    * Triggering execution by executing NETCONF get requests, extract parameters and trigger on values
-#    * Execute NETCONF get operation for operational or configuration data and return data
-#    * Generate a Syslog message with content passed to the function
-#    * Execute a CLI command on the device in exec mode and optionally return the results
-#    * Delay execution for a specified number of seconds
-#    * Decode a btrace log file, save the decoded file, find specified parameters in the log and return values
-#
-#  Revision History
-#    * 1.0 - 5.18.21 - petervh - Initial version
-#
-#################################################################################
+'''
+ ddrlib - Collection of functions used in DDR-Python scripts
+
+ ddrlib functions interact with device management interfaces including
+ NETCONF, ssh, cli, NETCONF RFC5277 notificaitons.
+ Scripts using ddrlib functions can perform multiple operations including:
+
+    * Triggering execution when specific Syslog messages are generated
+    * Triggering execution by executing show commands, extracting parameters and triggering on values
+    * Triggering execution by executing NETCONF get requests, extract parameters and trigger on values
+    * Execute NETCONF get operation for operational or configuration data and return data
+    * Generate a Syslog message with content passed to the function
+    * Execute a CLI command on the device in exec mode and optionally return the results
+    * Delay execution for a specified number of seconds
+    * Decode a btrace log file, save the decoded file, find specified parameters in the log and return values
+
+  ddrlib functions:
+
+    ddr_get_truth(data, relate, value, debug_flag)
+    ddr_nc_notify(device, access_type, triggers, return_response, debug_flag=False)
+    ddr_show_trigger(device, access_type, genie_parser_name, show_template, pcount, par1, par2, par3, parameter, relate, test_value, interval, retries, debug_flag=False)
+    ddr_nc_get_trigger(device, access_type, xpath, pcount, par1, par2, par3, leafs, relate, test_value, interval, retries, debug_flag=False)
+    ddr_nc_get(device, access_type, xpath, leaf, return_response, debug_flag=False)
+    ddr_nc_get_config(device, access_type, xpath, leaf, return_response, debug_flag=False)
+    ddr_nc_edit_config(device, access_type, datastore, commit, xpath, return_response, debug_flag=False)
+    ddr_nc_get_parameter(device, access_type, xpath, pcount, par1, par2, par3, leafs, return_value, debug_flag=False)
+    ddr_nc_get_config_parameter(device, access_type, xpath, pcount, par1, par2, par3, leafs, return_value, debug_flag=False)
+    ddr_write_to_syslog(syslog_message, debug_flag=False)
+    ddr_cli_command(device, access_type, command, return_response, timestamp, debug_flag=False)
+    ddr_wait(seconds, debug_flag=False)
+    ddr_cli_command(device, access_type, command, return_response, timestamp, debug_flag=False)
+    ddr_cli_configure(device, access_type, command, return_response, timestamp, debug_flag=False)
+    ddr_trigger_notify(device, triggers, debug_flag=False)
+    ddr_decode_btrace_log(device, access_type, show_template, pcount, par1, par2, par3, match_string, log_path, debug_flag=False)
+    ddr_control_file(control_file, delay)
+    ddr_show_parameter_all(device, access_type, genie_parser_name, show_template, pcount, par1, par2, par3, debug_flag=False)
+    
+  Revision History
+    * 0.1.43 - 7.1.22 - petervh - Updated ddr_decode_btrace_log for changes in btrace logging
+    * 1.0.04 - 9.6.22 - petervh - Update documentation, add ddr_ prefix to helper functions
+
+'''
 import pexpect
 import re
 import time
@@ -40,33 +61,140 @@ try:
     import cli
 except: pass
 
-def info_msg(message):
-  print("#### DDR Info: " + str(message))
+def ddr_info_msg(message):
+    """
+        This function writes an info level message to the DDR execution log
+        
+        :param message: Text to include in the DDR Info message
+        
+        Usage::
+
+              ddr_info_msg("Include this text in the log message")
+
+            :raises none:
+    """
+    print("#### DDR Info: " + str(message))
   
-def debug_msg(debug_flag, message):
-  if debug_flag:
-    print("**** DDR Debug: " + str(message))
+def ddr_debug_msg(debug_flag, message):
+    """
+        This function writes an debug level message to the DDR execution log if debug_flag = True
+        
+        :param debug_flag: True if message should be written to log
+        :param message: Text to include in the DDR Debug
+        Usage::
+
+              ddr_debug_msg(True, "Include this text in the log message")
+
+            :raises none:
+    """
+    if debug_flag:
+      print("**** DDR Debug: " + str(message))
     
-def warning_msg(message):
-  print("!!!! DDR Warning: " + str(message))
+def ddr_warning_msg(message):
+    """
+        This function writes an warning level message to the DDR execution log
+        
+        :param message: Text to include in the DDR warning message
+        
+        Usage::
 
-def error_msg(message):
-  print("%%%% DDR Error: " + str(message))
+              ddr_warning_msg("Include this text in the log message")
 
-def ddr_write_to_file(data: str, target_dir: str, filename: str, mode: str = "w"):
+            :raises none:
+    """
+    print("!!!! DDR Warning: " + str(message))
+
+def ddr_error_msg(message):
+    """
+        This function writes an error level message to the DDR execution log
+        
+        :param message: Text to include in the DDR error message
+        
+        Usage::
+
+              ddr_error_msg("Include this text in the log message")
+
+            :raises none:
+    """
+    print("%%%% DDR Error: " + str(message))
+
+def ddr_ddr_write_to_file(data: str, target_dir: str, filename: str, mode: str = "w"):
+    """
+        This function writes an informration to the DDR execution log
+        
+        :param data: String to include in the DDR Log
+        :param target_dir: String with directory path to the log: /bootflash/guest-share/ddr/ddr_high_cpu/
+        :param filename: String with name of the file in the target_dir
+        :param mode: Mode for writing to file, default value "w" is provided
+        
+        Usage::
+
+              ddr_write_to_file(command_result, TARGET_DIR, "ddr_high_cpu" + timestamp)
+
+            :raises none:
+    """
     with open(target_dir+filename, mode) as f:
         f.write(data)
 
 def ddr_delete_file(target_dir: str, filename: str):
+    """
+        This function deletes a file in the device guest-share
+        
+        :param target_dir: String with directory path to the log: /bootflash/guest-share/ddr/ddr_high_cpu/
+        
+        Usage::
+
+              ddr_delete_file(TARGET_DIR, "ddr_high_cpu_LOG")
+
+            :raises none:
+    """
     os.remove(target_dir+filename)
 
 def ddr_append_to_file(data: str, target_dir: str, filename: str):
+    """
+        This function appends informration to the DDR execution log
+        
+        :param data: String to include in the DDR Log
+        :param target_dir: String with directory path to the log: /bootflash/guest-share/ddr/ddr_high_cpu/
+        :param filename: String with name of the file in the target_dir
+        :param mode: Mode for appending to file, default value "a" is provided
+        
+        Usage::
+
+              ddr_append_to_file(command_result, TARGET_DIR, "ddr_high_cpu" + timestamp)
+
+            :raises none:
+    """
     ddr_write_to_file(data, target_dir, filename, "a")
 
 def ddr_log_data(data: str, target_dir: str,):
+    """
+        This function writes the results of CLI command execution to a dictionary "output.json"
+        
+        :param data: String to include in the CLI command execution
+        :param target_dir: String with directory path to the log: /bootflash/guest-share/ddr/ddr_high_cpu/
+        
+        Usage::
+
+              ddr_log_data(command_result, TARGET_DIR)
+
+            :raises none:
+    """
     ddr_append_to_file(data, target_dir, "output.json")
 
 def ddr_save_output(data: str, target_dir: str) -> None:
+    """
+        This function saves the results of all CLI command executions to a dictionary "output.json"
+        
+        :param data: String to include in the CLI command execution
+        :param target_dir: String with directory path to the log: /bootflash/guest-share/ddr/ddr_high_cpu/
+        
+        Usage::
+
+              ddr_save_output(command_results, TARGET_DIR)
+
+            :raises none:
+    """
     ddr_write_to_file(data, target_dir, "output.json", "w")
 
 def ddr_get_truth(data, relate, value, debug_flag): 
@@ -78,8 +206,12 @@ def ddr_get_truth(data, relate, value, debug_flag):
         :param relate: Using the "operator" python object operator.xx where xx is: eq, ne, le, ge, lt, gt
         :param value: Value to compare with the input "data"
     '''
-    result = relate(int(data), int(value))
-    debug_msg(debug_flag, f"ddr_get_truth: data_read: {str(data)} operator: {str(relate)} value: {str(value)} result: {str(result)}")
+    if isinstance(data, str):
+       result = relate(str(data), str(value))
+       debug_msg(debug_flag, f"ddr_get_truth: data_read: {str(data)} operator: {str(relate)} value: {str(value)} result: {str(result)}")
+    else:   
+        result = relate(int(data), int(value))
+        debug_msg(debug_flag, f"ddr_get_truth: data_read: {str(data)} operator: {str(relate)} value: {str(value)} result: {str(result)}")
     return result
 
 def ddr_nc_notify(device, access_type, triggers, return_response, debug_flag=False):
@@ -944,7 +1076,7 @@ def ddr_decode_btrace_log(device, access_type, show_template, pcount, par1, par2
                 
         Usage::
 
-          ddr_decode_btrace_log(device, "cli", "show platform software trace message {0} {1}", 2, "dmiauthd", "switch active R0", "none", "%DMI-5-AUTH_PASSED:", "/bootflash/guest-share/Btrace_dmiauthd_log", debug_flag=False)
+          ddr_decode_btrace_log(device, "cli", "show logging process {0} start last {1} minutes", 2, "dmiauthd", 20, "none", "%DMI-5-AUTH_PASSED:", "/bootflash/guest-share/Btrace_dmiauthd_log", debug_flag=False)
 
        :raises none:
 
@@ -1080,7 +1212,7 @@ def ddr_show_parameter_all(device, access_type, genie_parser_name, show_template
             This function executes a show command to read and return a dictionary of show command results
                 * Execute show command using python "cli" 
                 * Substitute parameters in the show command if required
-                * Return the show command parameters in the dict_name specified
+                * Return the show command parameters in a python dictionary
 
             :param device: Device information and credentials for running show command
             :param access_type: 'cli' to use the Python cli package in the guestshell
@@ -1120,6 +1252,7 @@ def ddr_show_parameter_all(device, access_type, genie_parser_name, show_template
         # If access-type is cli use the Python CLI package to run command
         #
           if str(access_type) == 'cli':
+            debug_msg(debug_flag, f" ddr_show_parameter_all: cli python command: {str(command)}")
             response = cli.cli(command)
             debug_msg(debug_flag, f" ddr_show_parameter_all: cli python response: {str(response)}")
 
@@ -1130,7 +1263,7 @@ def ddr_show_parameter_all(device, access_type, genie_parser_name, show_template
     # parse the response to get the test leaf value
     #
           parser = genie_str_to_class(genie_parser_name)
-          dictionary = parser.parser(output=response)
+          dictionary = parser.cli(output=response)
           debug_msg(debug_flag, f"ddr_show_parameter_all: parser dictionary: {str(dictionary)}")
           return dictionary
 
